@@ -92,7 +92,7 @@ static int autofs_check_fd_stat(struct stat *stat, int prgp, int fd,
 		return 0;
 	if (stat->st_ino != ino)
 		return 0;
-	if (parse_fdinfo_pid(prgp, fd, FD_TYPES__UND, NULL, &fdinfo))
+	if (parse_fdinfo_pid(prgp, fd, FD_TYPES__UND, &fdinfo))
 		return -1;
 
 	*mode = fdinfo.flags & O_WRONLY;
@@ -359,11 +359,9 @@ static int access_autofs_mount(struct mount_info *pm)
 	if (new_pid_ns < 0)
 		return -1;
 
-	old_pid_ns = open("/proc/self/ns/pid", O_RDONLY);
-	if (old_pid_ns < 0) {
-		pr_perror("Can't open /proc/self/ns/pid");
+	old_pid_ns = open_proc(PROC_SELF, "ns/pid");
+	if (old_pid_ns < 0)
 		goto close_new_pid_ns;
-	}
 
 	if (switch_ns(pm->nsid->ns_pid, &mnt_ns_desc, &old_mnt_ns)) {
 		pr_err("failed to switch to mount namespace\n");
@@ -533,6 +531,8 @@ typedef struct autofs_info_s {
 	AutofsEntry *entry;
 	char *mnt_path;
 	dev_t mnt_dev;
+	struct mount_info *mi;
+	struct pprep_head ph;
 } autofs_info_t;
 
 static int dup_pipe_info(struct pipe_info *pi, int flags,
@@ -929,9 +929,10 @@ static int autofs_create_pipe(struct pstree_item *task, autofs_info_t *i,
 	return autofs_create_fle(task, fe, &i->pi.d);
 }
 
-static int autofs_add_mount_info(void *data)
+static int autofs_add_mount_info(struct pprep_head *ph)
 {
-	struct mount_info *mi = data;
+	autofs_info_t *ai = container_of(ph, autofs_info_t, ph);
+	struct mount_info *mi = ai->mi;
 	autofs_info_t *info = mi->private;
 	AutofsEntry *entry = info->entry;
 	autofs_info_t *i;
@@ -1065,11 +1066,11 @@ int autofs_mount(struct mount_info *mi, const char *source, const
 
 	/* Otherwise we have to add shared object creation callback */
 	if (entry->fd != AUTOFS_CATATONIC_FD) {
-		ret = add_post_prepare_cb(autofs_add_mount_info, mi);
-		if (ret < 0)
-			goto free_info;
+		info->ph.actor = autofs_add_mount_info;
+		add_post_prepare_cb(&info->ph);
 	}
 
+	info->mi = mi;
 	mi->private = info;
 
 free_opts:
