@@ -7,7 +7,7 @@
 #include "imgset.h"
 #include "kcmp.h"
 #include "pstree.h"
-#include "ptrace.h"
+#include <compel/ptrace.h>
 #include "proc_parse.h"
 #include "restorer.h"
 #include "seccomp.h"
@@ -49,15 +49,15 @@ static int collect_filter_for_pstree(struct pstree_item *item)
 	struct sock_filter buf[BPF_MAXINSNS];
 	void *m;
 
-	if (item->pid.state == TASK_DEAD ||
-	    dmpi(item)->pi_creds->seccomp_mode != SECCOMP_MODE_FILTER)
+	if (item->pid->state == TASK_DEAD ||
+	    dmpi(item)->pi_creds->s.seccomp_mode != SECCOMP_MODE_FILTER)
 		return 0;
 
 	for (i = 0; true; i++) {
 		int len;
 		struct seccomp_info *info, *inherited = NULL;
 
-		len = ptrace(PTRACE_SECCOMP_GET_FILTER, item->pid.real, i, buf);
+		len = ptrace(PTRACE_SECCOMP_GET_FILTER, item->pid->real, i, buf);
 		if (len < 0) {
 			if (errno == ENOENT) {
 				/* end of the search */
