@@ -1,9 +1,18 @@
+function exit_hook()
+{
+	test -z "$GCOV" && return
+	make gcov
+}
+
 function prep()
 {
+	test -n "$SKIP_PREP" && return
 	# systemd executes jenkins in a separate sched cgroup.
 	echo 950000 > /sys/fs/cgroup/cpu,cpuacct/system/cpu.rt_runtime_us || true
 	echo 950000 > /sys/fs/cgroup/cpu,cpuacct/system/jenkins.service/cpu.rt_runtime_us || true
 
+	test -n "$GCOV" && umask 0000
+
 	ulimit -c unlimited &&
 	export CFLAGS=-g
 	git clean -dfx &&
@@ -11,12 +20,12 @@ function prep()
 	make -j 4 -C test/zdtm/ &&
 	make -C test zdtm_ct &&
 	mkdir -p test/report &&
-	true
+	trap exit_hook EXIT
 }
 
 function mount_tmpfs_to_dump()
 {
-		
+	test -n "$SKIP_PREP" && return	
 	mkdir -p test/dump &&
 	mount -t tmpfs criu_dump test/dump &&
 	true
